==> v1.13.3.tar.gz: wcs/wcs/qommon/sessions.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import time

from quixote import get_publisher

from quixote.session import Session as QuixoteSession
from quixote.session import SessionManager as QuixoteSessionManager
from quixote.util import randbytes

from qommon.storage import StorableObject
from publisher import get_cfg
from quixote.publish import get_session, get_session_manager, get_request

class QommonSession(QuixoteSession):
    def __init__(self, id):
        QuixoteSession.__init__(self, id)
        env = get_request().environ
        # add support for X_FORWARDED_FOR
        self.__remote_address = env.get('X_FORWARDED_FOR', env.get('REMOTE_ADDR'))

class CaptchaSession(object):
    MAX_CAPTCHA_TOKENS = 8
    _captcha_tokens = None
    won_captcha = False

    def create_captcha_token(self):
        if not self._captcha_tokens:
            self._captcha_tokens = []
        token = {'token': randbytes(8), 'answer': None}
        self._captcha_tokens.append(token)
        extra = len(self._captcha_tokens) - self.MAX_CAPTCHA_TOKENS
        if extra > 0:
            del self._captcha_tokens[:extra]
        return token

    def get_captcha_token(self, token):
        if not self._captcha_tokens:
            return None
        try:
            return [x for x in self._captcha_tokens if x.get('token') == token][0]
        except IndexError:
            return None

    def remove_captcha_token(self, token):
        token = self.get_captcha_token(token)
        if token:
            self._captcha_tokens.remove(token)

    def has_info(self):
        return self.won_captcha or self._captcha_tokens

class Session(QommonSession, CaptchaSession, StorableObject):
    _names = 'sessions'

    name_identifier = None
    after_url = None
    lasso_session_dump = None
    lasso_session_index = None
    lasso_anonymous_identity_dump = None
    lasso_identity_provider_id = None
    lasso_manage_name_id_dump = None
    message = None
    saml_idp_cookie = None
    ident_idp_token = None
    tempfiles = None
    jsonp_display_values = None
    extra_variables = None

    username = None # only set on password authentication

    def has_info(self):
        return self.name_identifier or self.after_url or \
            self.lasso_session_dump or self.message or \
            self.lasso_anonymous_identity_dump or \
            self.lasso_manage_name_id_dump or \
            self.lasso_identity_provider_id or \
            self.ident_idp_token or \
            self.tempfiles or \
            self.jsonp_display_values or \
            self.extra_variables or \
            CaptchaSession.has_info(self) or \
            QuixoteSession.has_info(self)
    is_dirty = has_info

    def get_session_id(self):
        return self.id

    def set_session_id(self, session_id):
        self.id = session_id

    session_id = property(get_session_id, set_session_id)

    def remove_form_token(self, token):
        super(Session, self).remove_form_token(token)
        self.store()

    def get_user(self):
        user_id = QuixoteSession.get_user(self)
        if user_id:
            if str(user_id).startswith('anonymous-'):
                user = get_publisher().user_class()
                user.id = user_id
                user.anonymous = True
                user.name_identifiers = [ self.name_identifier ]
                user.lasso_dump = self.lasso_anonymous_identity_dump
                return user
            else:
                try:
                    return get_publisher().user_class.get(user_id)
                except KeyError:
                    pass
        return None

    def set_user(self, user_id):
        QuixoteSession.set_user(self, user_id)
        if str(user_id).startswith('anonymous-'):
            # do not store connection time for anonymous users
            return
        try:
            user = get_publisher().user_class.get(user_id)
            user.last_seen = time.time()
            user.store()
        except KeyError:
            pass

    def display_message(self):
        if not self.message or not self.message[1]:
            return ''
        from quixote.html import htmltext
        s = htmltext('<div class="%snotice">%s</div>' % self.message)
        self.message = None
        return s

    def get_user_object(self):
        return self.get_user()

    def add_tempfile(self, upload):
        from wcs.qommon.form import PicklableUpload
        token = randbytes(8)
        upload.__class__ = PicklableUpload
        upload.time = time.time()
        upload.token = token
        # and saves it
        dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
        if not os.path.exists(dirname):
            os.mkdir(dirname)
        filename = os.path.join(dirname, token)
        fd = open(filename, 'w')
        upload.get_file_pointer().seek(0)
        fd.write(upload.get_file_pointer().read())
        size = fd.tell()
        fd.close()
        if not self.tempfiles:
            self.tempfiles = {}
        self.tempfiles[token] = {
            'orig_filename': upload.orig_filename,
            'base_filename': upload.base_filename,
            'content_type': upload.content_type,
            'charset': upload.charset,
            'size': size,
        }
        return token

    def clean_tempfiles(self):
        dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
        if not self.tempfiles:
            return
        for token in self.tempfiles:
            filename = os.path.join(dirname, token)
            try:
                os.unlink(filename)
            except OSError:
                pass

    def get_tempfile(self, token):
        if not self.tempfiles:
            return None
        if not token:
            return None
        return self.tempfiles.get(token)

    def get_tempfile_content(self, token):
        temp = self.tempfiles.get(token)

        from wcs.qommon.form import PicklableUpload
        value = PicklableUpload(temp['orig_filename'],
                        temp['content_type'], temp['charset'])
        dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
        filename = os.path.join(dirname, token)
        value.token = token
        value.fp = open(filename)
        return value

    def add_extra_variables(self, **kwargs):
        if not self.extra_variables:
            self.extra_variables = {}
        self.extra_variables.update(kwargs)

    def get_substitution_variables(self, prefix='session_var_'):
        d = {}
        if self.extra_variables:
            for k, v in self.extra_variables.items():
                d[prefix + k] = v
        return d


class QommonSessionManager(QuixoteSessionManager):
    def start_request(self):
        QuixoteSessionManager.start_request(self)
        session_cfg = get_cfg('session', {})
        ip_linked = session_cfg.get('ip_linked', False)
        if ip_linked:
            env = get_request().environ
            ip = env.get('X_FORWARDED_FOR', env.get('REMOTE_ADDR'))
            session = get_session()
            if ip != session._remote_address:
                # clean cookie
                get_session_manager().expire_session()
                # start a new cookie
                QuixoteSessionManager.start_request(self)
                session = get_session()

class StorageSessionManager(QommonSessionManager):
    def __init__(self):
        QuixoteSessionManager.__init__(self, session_class=BasicSession)

    def forget_changes(self, session):
        pass

    def __getitem__(self, session_id):
        try:
            return BasicSession.get(session_id)
        except KeyError:
            raise KeyError

    def get(self, session_id, default = None):
        try:
            return BasicSession.get(session_id)
        except KeyError:
            return default
        except ValueError: # happens for "insecure string pickle"
            return default

    def commit_changes(self, session):
        if session and session.id:
            session.store()

    def keys(self):
        return BasicSession.keys()

    def values(self):
        return BasicSession.values()

    def items(self):
        return BasicSession.items()

    def has_key(self, session_id):
        return BasicSession.has_key(session_id)

    def __setitem__(self, session_id, session):
        session.store()

    def __delitem__(self, session_id):
        if not session_id:
            return
        try:
            session = self.session_class.get(session_id)
        except KeyError:
            session = None
        try:
            self.session_class.remove_object(session_id)
        except OSError:
            raise KeyError

        if session:
            session.clean_tempfiles()

    def get_sessions_for_saml(self, name_identifier = Ellipsis, \
            session_indexes = ()):
        ret = (x for x in self.values() \
                if (not session_indexes \
                    or x.lasso_session_index in session_indexes) \
                   and name_identifier in (x.name_identifier or []))
        return ret

    def get_session_for_saml(self, name_identifier = None, session_index = None):
        if session_index:
            session_indexes = (session_index,)
        else:
            session_indexes = ()
        for session in self.get_sessions_for_saml(name_identifier,
                session_indexes):
            return session
        return None

==> v1.88.tar.gz: wcs/wcs/qommon/sessions.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import time

from quixote import get_publisher

from quixote.session import Session as QuixoteSession
from quixote.session import SessionManager as QuixoteSessionManager
from quixote.util import randbytes

from qommon.storage import StorableObject
from publisher import get_cfg
from quixote.publish import get_session, get_session_manager, get_request

class QommonSession(QuixoteSession):
    def __init__(self, id):
        QuixoteSession.__init__(self, id)
        env = get_request().environ
        # add support for X_FORWARDED_FOR
        self.__remote_address = env.get('X_FORWARDED_FOR', env.get('REMOTE_ADDR'))

class CaptchaSession(object):
    MAX_CAPTCHA_TOKENS = 8
    _captcha_tokens = None
    won_captcha = False

    def create_captcha_token(self):
        if not self._captcha_tokens:
            self._captcha_tokens = []
        token = {'token': randbytes(8), 'answer': None}
        self._captcha_tokens.append(token)
        extra = len(self._captcha_tokens) - self.MAX_CAPTCHA_TOKENS
        if extra > 0:
            del self._captcha_tokens[:extra]
        return token

    def get_captcha_token(self, token):
        if not self._captcha_tokens:
            return None
        try:
            return [x for x in self._captcha_tokens if x.get('token') == token][0]
        except IndexError:
            return None

    def remove_captcha_token(self, token):
        token = self.get_captcha_token(token)
        if token:
            self._captcha_tokens.remove(token)

    def has_info(self):
        return self.won_captcha or self._captcha_tokens

class Session(QommonSession, CaptchaSession, StorableObject):
    _names = 'sessions'

    name_identifier = None
    lasso_session_dump = None
    lasso_session_index = None
    lasso_anonymous_identity_dump = None
    lasso_identity_provider_id = None
    lasso_manage_name_id_dump = None
    message = None
    saml_authn_context = None
    saml_idp_cookie = None
    ident_idp_token = None
    tempfiles = None
    jsonp_display_values = None
    extra_variables = None
    expire = None

    username = None # only set on password authentication

    def set_expire(self, expire):
        self.expire = expire

    def set_duration(self, duration):
        self.set_expire(time.time() + duration)

    def is_expired(self):
        if self.expire:
            return time.time() >= self.expire
        duration = get_publisher().get_site_option('session_max_age')
        if duration is None:
            return False
        try:
            duration = int(duration)
        except ValueError:
            return False
        return (time.time() - self.get_access_time()) > duration

    def has_info(self):
        return self.name_identifier or \
            self.lasso_session_dump or self.message or \
            self.lasso_anonymous_identity_dump or \
            self.lasso_manage_name_id_dump or \
            self.lasso_identity_provider_id or \
            self.saml_authn_context or \
            self.ident_idp_token or \
            self.tempfiles or \
            self.jsonp_display_values or \
            self.extra_variables or \
            CaptchaSession.has_info(self) or \
            self.expire or \
            QuixoteSession.has_info(self)
    is_dirty = has_info

    def get_session_id(self):
        return self.id

    def set_session_id(self, session_id):
        self.id = session_id

    session_id = property(get_session_id, set_session_id)

    def remove_form_token(self, token):
        super(Session, self).remove_form_token(token)
        self.store()

    def get_user(self):
        user_id = QuixoteSession.get_user(self)
        if user_id:
            if str(user_id).startswith('anonymous-'):
                user = get_publisher().user_class()
                user.id = user_id
                user.anonymous = True
                user.name_identifiers = [ self.name_identifier ]
                user.lasso_dump = self.lasso_anonymous_identity_dump
                return user
            else:
                try:
                    return get_publisher().user_class.get(user_id)
                except KeyError:
                    pass
        return None

    def set_user(self, user_id):
        self.id = None # force a new session id to be assigned
        QuixoteSession.set_user(self, user_id)
        if str(user_id).startswith('anonymous-'):
            # do not store connection time for anonymous users
            return
        try:
            user = get_publisher().user_class.get(user_id)
            user.last_seen = time.time()
            user.store()
        except KeyError:
            pass

    def display_message(self):
        if not self.message or not self.message[1]:
            return ''
        from quixote.html import htmltext
        s = htmltext('<div id="messages"><ul class="messages">'
                     '<li class="%s">%s</li></ul></div>') % self.message
        self.message = None
        return s

    def get_user_object(self):
        return self.get_user()

    def get_authentication_context(self):
        for context in get_publisher().get_supported_authentication_contexts():
            contexts = get_publisher().get_authentication_saml_contexts(context)
            if self.saml_authn_context in contexts:
                return context
        return None

    def add_tempfile(self, upload):
        from wcs.qommon.form import PicklableUpload
        token = randbytes(8)
        upload.__class__ = PicklableUpload
        upload.time = time.time()
        upload.token = token
        # and saves it
        dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
        if not os.path.exists(dirname):
            os.mkdir(dirname)
        filename = os.path.join(dirname, token)
        fd = open(filename, 'w')
        upload.get_file_pointer().seek(0)
        fd.write(upload.get_file_pointer().read())
        size = fd.tell()
        fd.close()
        if not self.tempfiles:
            self.tempfiles = {}
        self.tempfiles[token] = {
            'orig_filename': upload.orig_filename,
            'base_filename': upload.base_filename,
            'content_type': upload.content_type,
            'charset': upload.charset,
            'size': size,
        }
        return token

    def clean_tempfiles(self):
        dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
        if not self.tempfiles:
            return
        for token in self.tempfiles:
            filename = os.path.join(dirname, token)
            try:
                os.unlink(filename)
            except OSError:
                pass

    def get_tempfile(self, token):
        if not self.tempfiles:
            return None
        if not token:
            return None
        return self.tempfiles.get(token)

    def get_tempfile_content(self, token):
        temp = self.tempfiles.get(token)

        from wcs.qommon.form import PicklableUpload
        value = PicklableUpload(temp['orig_filename'],
                        temp['content_type'], temp['charset'])
        dirname = os.path.join(get_publisher().app_dir, 'tempfiles')
        filename = os.path.join(dirname, token)
        value.token = token
        value.fp = open(filename)
        return value

    def add_extra_variables(self, **kwargs):
        if not self.extra_variables:
            self.extra_variables = {}
        self.extra_variables.update(kwargs)

    def get_substitution_variables(self, prefix='session_var_'):
        d = {}
        if self.extra_variables:
            for k, v in self.extra_variables.items():
                d[prefix + k] = v
        return d


class QommonSessionManager(QuixoteSessionManager):
    def start_request(self):
        QuixoteSessionManager.start_request(self)
        session_cfg = get_cfg('session', {})
        ip_linked = session_cfg.get('ip_linked', False)
        if ip_linked:
            env = get_request().environ
            ip = env.get('X_FORWARDED_FOR', env.get('REMOTE_ADDR'))
            session = get_session()
            if ip != session._remote_address:
                # clean cookie
                get_session_manager().expire_session()
                # start a new cookie
                QuixoteSessionManager.start_request(self)
                session = get_session()

class StorageSessionManager(QommonSessionManager):
    def __init__(self):
        QuixoteSessionManager.__init__(self, session_class=BasicSession)

    def forget_changes(self, session):
        pass

    def __getitem__(self, session_id):
        try:
            session = BasicSession.get(session_id)
            if session.is_expired():
                try:
                    session.remove_self()
                except OSError:
                    pass
                raise KeyError
            return session
        except KeyError:
            raise KeyError

    def get(self, session_id, default = None):
        try:
            return self[session_id]
        except KeyError:
            return default
        except ValueError: # happens for "insecure string pickle"
            return default

    def commit_changes(self, session):
        if session and session.id:
            session.store()

    def keys(self):
        return BasicSession.keys()

    def values(self):
        return BasicSession.values()

    def items(self):
        return BasicSession.items()

    def has_key(self, session_id):
        return BasicSession.has_key(session_id)

    def __setitem__(self, session_id, session):
        session.store()

    def __delitem__(self, session_id):
        if not session_id:
            return
        try:
            session = self.session_class.get(session_id)
        except KeyError:
            session = None
        try:
            self.session_class.remove_object(session_id)
        except OSError:
            raise KeyError

        if session:
            session.clean_tempfiles()

    def get_sessions_for_saml(self, name_identifier = Ellipsis, \
            session_indexes = ()):
        ret = (x for x in self.values() \
                if (not session_indexes \
                    or x.lasso_session_index in session_indexes) \
                   and name_identifier in (x.name_identifier or []))
        return ret

    def get_session_for_saml(self, name_identifier = None, session_index = None):
        if session_index:
            session_indexes = (session_index,)
        else:
            session_indexes = ()
        for session in self.get_sessions_for_saml(name_identifier,
                session_indexes):
            return session
        return None
